@@ -54,11 +54,14 @@ static PangoAttribute *pango_attr_size_new_internal (int                   size,
 						     gboolean              absolute);
 
 
+static GHashTable *name_map = NULL;
+
 /**
  * pango_attr_type_register:
- * @name: an identifier for the type (currently unused.)
+ * @name: an identifier for the type
  *
- * Allocate a new attribute type ID.
+ * Allocate a new attribute type ID.  The attribute type name can be accessed
+ * later by using pango_attr_type_get_name().
  *
  * Return value: the new type ID.
  **/
@@ -66,8 +69,66 @@ PangoAttrType
 pango_attr_type_register (const gchar *name)
 {
   static guint current_type = 0x1000000;
+  guint type = current_type++;
 
-  return current_type++;
+  if (name)
+    {
+      if (G_UNLIKELY (!name_map))
+	name_map = g_hash_table_new (NULL, NULL);
+
+      g_hash_table_insert (name_map, GUINT_TO_POINTER (type), (gpointer) g_intern_string (name));
+    }
+
+  return type;
+}
+
+/**
+ * pango_attr_type_get_name:
+ * @type: an attribute type ID to fetch the name for
+ *
+ * Fetches the attribute type name passed in when registering the type using
+ * pango_attr_type_register().
+ *
+ * The returned value is an interned string (see g_intern_string() for what
+ * that means) that should not be modified or freed.
+ *
+ * Return value: the type ID name (which may be %NULL), or %NULL if @type is
+ * a built-in Pango attribute type or invalid. 
+ *
+ * Since: 1.22
+ **/
+G_CONST_RETURN char *
+pango_attr_type_get_name (PangoAttrType type)
+{
+  const char *result = NULL;
+
+  if (name_map)
+    result = g_hash_table_lookup (name_map, GUINT_TO_POINTER ((guint) type));
+
+  return result;
+}
+
+/**
+ * pango_attribute_init:
+ * @attr: a #PangoAttribute
+ * @klass: a #PangoAttributeClass
+ *
+ * Initializes @attr's klass to @klass, it's start_index to zero,
+ * and end_index to %G_MAXUINT such that the attribute applies
+ * to the entire text by default.
+ *
+ * Since: 1.20
+ **/
+void
+pango_attribute_init (PangoAttribute       *attr,
+		      const PangoAttrClass *klass)
+{
+  g_return_if_fail (attr != NULL);
+  g_return_if_fail (klass != NULL);
+
+  attr->klass = klass;
+  attr->start_index = 0;
+  attr->end_index = G_MAXUINT;
 }
 
 /**
@@ -158,8 +219,7 @@ pango_attr_string_new (const PangoAttrClass *klass,
 		       const char           *str)
 {
   PangoAttrString *result = g_slice_new (PangoAttrString);
-
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = g_strdup (str);
 
   return (PangoAttribute *)result;
@@ -231,11 +291,8 @@ pango_attr_language_new (PangoLanguage *language)
     pango_attr_language_equal
   };
 
-  g_return_val_if_fail (language != NULL, NULL);
-
   result = g_slice_new (PangoAttrLanguage);
-
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->value = language;
 
   return (PangoAttribute *)result;
@@ -279,7 +336,7 @@ pango_attr_color_new (const PangoAttrClass *klass,
 		      guint16               blue)
 {
   PangoAttrColor *result = g_slice_new (PangoAttrColor);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->color.red = red;
   result->color.green = green;
   result->color.blue = blue;
@@ -370,7 +427,7 @@ pango_attr_int_new (const PangoAttrClass *klass,
 		    int                   value)
 {
   PangoAttrInt *result = g_slice_new (PangoAttrInt);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = value;
 
   return (PangoAttribute *)result;
@@ -407,7 +464,7 @@ pango_attr_float_new  (const PangoAttrClass *klass,
 		       double                value)
 {
   PangoAttrFloat *result = g_slice_new (PangoAttrFloat);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = value;
 
   return (PangoAttribute *)result;
@@ -462,7 +519,7 @@ pango_attr_size_new_internal (int size,
   };
 
   result = g_slice_new (PangoAttrSize);
-  result->attr.klass = absolute ? &absolute_klass : &klass;
+  pango_attribute_init (&result->attr, absolute ? &absolute_klass : &klass);
   result->size = size;
   result->absolute = absolute;
 
@@ -640,7 +697,7 @@ pango_attr_font_desc_new (const PangoFontDescription *desc)
   };
 
   PangoAttrFontDesc *result = g_slice_new (PangoAttrFontDesc);
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->desc = pango_font_description_copy (desc);
 
   return (PangoAttribute *)result;
@@ -936,7 +993,7 @@ pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
   g_return_val_if_fail (logical_rect != NULL, NULL);
 
   result = g_slice_new (PangoAttrShape);
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->ink_rect = *ink_rect;
   result->logical_rect = *logical_rect;
   result->data = data;
@@ -1030,7 +1087,7 @@ pango_attr_list_get_type (void)
 {
   static GType our_type = 0;
 
-  if (our_type == 0)
+  if (G_UNLIKELY (our_type == 0))
     our_type = g_boxed_type_register_static (I_("PangoAttrList"),
 					     (GBoxedCopyFunc) pango_attr_list_copy,
 					     (GBoxedFreeFunc) pango_attr_list_unref);
@@ -1060,7 +1117,7 @@ pango_attr_list_new (void)
 
 /**
  * pango_attr_list_ref:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Increase the reference count of the given attribute list by one.
  *
@@ -1071,7 +1128,8 @@ pango_attr_list_new (void)
 PangoAttrList *
 pango_attr_list_ref (PangoAttrList *list)
 {
-  g_return_val_if_fail (list != NULL, NULL);
+  if (list == NULL)
+    return NULL;
 
   list->ref_count++;
 
@@ -1080,7 +1138,7 @@ pango_attr_list_ref (PangoAttrList *list)
 
 /**
  * pango_attr_list_unref:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Decrease the reference count of the given attribute list by one.
  * If the result is zero, free the attribute list and the attributes
@@ -1091,7 +1149,9 @@ pango_attr_list_unref (PangoAttrList *list)
 {
   GSList *tmp_list;
 
-  g_return_if_fail (list != NULL);
+  if (list == NULL)
+    return;
+
   g_return_if_fail (list->ref_count > 0);
 
   list->ref_count--;
@@ -1114,13 +1174,14 @@ pango_attr_list_unref (PangoAttrList *list)
 
 /**
  * pango_attr_list_copy:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Copy @list and return an identical new list.
  *
  * Return value: the newly allocated #PangoAttrList, with a
  *               reference count of one, which should
  *               be freed with pango_attr_list_unref().
+ *               Returns %NULL if @list was %NULL.
  **/
 PangoAttrList *
 pango_attr_list_copy (PangoAttrList *list)
@@ -1129,7 +1190,8 @@ pango_attr_list_copy (PangoAttrList *list)
   GSList *iter;
   GSList *new_attrs;
 
-  g_return_val_if_fail (list != NULL, NULL);
+  if (list == NULL)
+    return NULL;
 
   new = pango_attr_list_new ();
 
@@ -1166,7 +1228,7 @@ pango_attr_list_insert_internal (PangoAttrList  *list,
   else if (((PangoAttribute *)list->attributes_tail->data)->start_index < start_index ||
 	   (!before && ((PangoAttribute *)list->attributes_tail->data)->start_index == start_index))
     {
-      g_slist_append (list->attributes_tail, attr);
+      list->attributes_tail = g_slist_append (list->attributes_tail, attr);
       list->attributes_tail = list->attributes_tail->next;
       g_assert (list->attributes_tail);
     }
