@@ -24,6 +24,44 @@
 #include "pango-ot-private.h"
 #include "pangofc-private.h"
 
+/* cache a single HB_Buffer */
+static HB_Buffer cached_buffer = NULL;
+G_LOCK_DEFINE_STATIC (cached_buffer);
+
+static HB_Buffer
+acquire_buffer (gboolean *free_buffer)
+{
+  HB_Buffer buffer;
+
+  if (G_LIKELY (G_TRYLOCK (cached_buffer)))
+    {
+      if (G_UNLIKELY (!cached_buffer))
+	hb_buffer_new (&cached_buffer);
+
+      buffer = cached_buffer;
+      *free_buffer = FALSE;
+    }
+  else
+    {
+      hb_buffer_new (&buffer);
+      *free_buffer = TRUE;
+    }
+
+  return buffer;
+}
+
+static void
+release_buffer (HB_Buffer buffer, gboolean free_buffer)
+{
+  if (G_LIKELY (!free_buffer))
+    {
+      hb_buffer_clear (buffer);
+      G_UNLOCK (cached_buffer);
+    }
+  else
+    hb_buffer_free (buffer);
+}
+
 /**
  * pango_ot_buffer_new
  * @font: a #PangoFcFont
@@ -38,25 +76,14 @@
 PangoOTBuffer *
 pango_ot_buffer_new (PangoFcFont *font)
 {
-  /* We lock the font here immediately for the silly reason
-   * of getting the FT_Memory; otherwise we'd have to
-   * add a new operation to PangoFcFontmap; callers will
-   * probably already have the font locked, however,
-   * so there is little performance penalty.
-   */
   PangoOTBuffer *buffer = g_slice_new (PangoOTBuffer);
-  FT_Face face = pango_fc_font_lock_face (font);
-
-  if (hb_buffer_new (face->memory, &buffer->buffer) != FT_Err_Ok)
-    g_warning ("Allocation of HB_Buffer failed"); /* this doesn't happen */
 
+  buffer->buffer = acquire_buffer (&buffer->should_free_hb_buffer);
   buffer->font = g_object_ref (font);
   buffer->applied_gpos = FALSE;
   buffer->rtl = FALSE;
   buffer->zero_width_marks = FALSE;
 
-  pango_fc_font_unlock_face (font);
-
   return buffer;
 }
 
@@ -71,7 +98,7 @@ pango_ot_buffer_new (PangoFcFont *font)
 void
 pango_ot_buffer_destroy (PangoOTBuffer *buffer)
 {
-  hb_buffer_free (buffer->buffer);
+  release_buffer (buffer->buffer, buffer->should_free_hb_buffer);
   g_object_unref (buffer->font);
   g_slice_free (PangoOTBuffer, buffer);
 }
@@ -335,11 +362,11 @@ pango_ot_buffer_output (const PangoOTBuffer *buffer,
 	{
 	  PangoRectangle logical_rect;
 
-	  FT_UShort property;
+	  HB_UShort property;
 
 	  if (buffer->zero_width_marks &&
 	      gdef &&
-	      HB_GDEF_Get_Glyph_Property (gdef, glyphs->glyphs[i].glyph, &property) == FT_Err_Ok &&
+	      HB_GDEF_Get_Glyph_Property (gdef, glyphs->glyphs[i].glyph, &property) == HB_Err_Ok &&
 	      (property == HB_GDEF_MARK || (property & HB_LOOKUP_FLAG_IGNORE_SPECIAL_MARKS) != 0))
 	    {
 	      glyphs->glyphs[i].geometry.width = 0;
