@@ -74,6 +74,10 @@ static void                  pango_fc_font_set_property (GObject          *objec
 							 guint             prop_id,
 							 const GValue     *value,
 							 GParamSpec       *pspec);
+static void                  pango_fc_font_get_property (GObject          *object,
+							 guint             prop_id,
+							 GValue           *value,
+							 GParamSpec       *pspec);
 static PangoEngineShape *    pango_fc_font_find_shaper  (PangoFont        *font,
 							 PangoLanguage    *language,
 							 guint32           ch);
@@ -89,7 +93,7 @@ static PangoFontDescription *pango_fc_font_describe_absolute (PangoFont        *
 #define PANGO_FC_FONT_LOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->lock_face (font))
 #define PANGO_FC_FONT_UNLOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->unlock_face (font))
 
-G_DEFINE_TYPE (PangoFcFont, pango_fc_font, PANGO_TYPE_FONT)
+G_DEFINE_ABSTRACT_TYPE (PangoFcFont, pango_fc_font, PANGO_TYPE_FONT)
 
 static void
 pango_fc_font_class_init (PangoFcFontClass *class)
@@ -103,6 +107,7 @@ pango_fc_font_class_init (PangoFcFontClass *class)
 
   object_class->finalize = pango_fc_font_finalize;
   object_class->set_property = pango_fc_font_set_property;
+  object_class->get_property = pango_fc_font_get_property;
   font_class->describe = pango_fc_font_describe;
   font_class->describe_absolute = pango_fc_font_describe_absolute;
   font_class->find_shaper = pango_fc_font_find_shaper;
@@ -114,7 +119,8 @@ pango_fc_font_class_init (PangoFcFontClass *class)
 				   g_param_spec_pointer ("pattern",
 							 "Pattern",
 							 "The fontconfig pattern for this font",
-							 G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
+							 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+							 G_PARAM_STATIC_STRINGS));
 
   g_type_class_add_private (object_class, sizeof (PangoFcFontPrivate));
 }
@@ -220,6 +226,26 @@ pango_fc_font_set_property (GObject       *object,
     }
 }
 
+static void
+pango_fc_font_get_property (GObject       *object,
+			    guint          prop_id,
+			    GValue        *value,
+			    GParamSpec    *pspec)
+{
+  switch (prop_id)
+    {
+    case PROP_PATTERN:
+      {
+	PangoFcFont *fcfont = PANGO_FC_FONT (object);
+	g_value_set_pointer (value, fcfont->font_pattern);
+      }
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static PangoFontDescription *
 pango_fc_font_describe (PangoFont *font)
 {
@@ -304,7 +330,7 @@ get_face_metrics (PangoFcFont      *fcfont,
   TT_OS2 *os2;
   gboolean have_transform = FALSE;
 
-  if (!face)
+  if (G_UNLIKELY (!face))
     {
       metrics->descent = 0;
       metrics->ascent = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_HEIGHT;
@@ -358,27 +384,32 @@ get_face_metrics (PangoFcFont      *fcfont,
       metrics->ascent = PANGO_UNITS_26_6 (ascender);
     }
 
-  /* Versions of FreeType < 2.1.8 get underline thickness wrong
-   * for Postscript fonts (always zero), so we need a fallback
-   */
-  if (face->underline_thickness == 0)
+
+  metrics->underline_thickness = 0;
+  metrics->underline_position = 0;
+
+  {
+    FT_Fixed ft_thickness, ft_position;
+
+    ft_thickness = FT_MulFix (face->underline_thickness, face->size->metrics.y_scale);
+    metrics->underline_thickness = PANGO_UNITS_26_6 (ft_thickness);
+
+    ft_position = FT_MulFix (face->underline_position, face->size->metrics.y_scale);
+    metrics->underline_position = PANGO_UNITS_26_6 (ft_position);
+  }
+
+  if (metrics->underline_thickness == 0 || metrics->underline_position == 0)
     {
       metrics->underline_thickness = (PANGO_SCALE * face->size->metrics.y_ppem) / 14;
       metrics->underline_position = - metrics->underline_thickness;
     }
-  else
-    {
-      FT_Fixed ft_thickness, ft_position;
 
-      ft_thickness = FT_MulFix (face->underline_thickness, face->size->metrics.y_scale);
-      metrics->underline_thickness = PANGO_UNITS_26_6 (ft_thickness);
 
-      ft_position = FT_MulFix (face->underline_position, face->size->metrics.y_scale);
-      metrics->underline_position = PANGO_UNITS_26_6 (ft_position);
-    }
+  metrics->strikethrough_thickness = 0;
+  metrics->strikethrough_position = 0;
 
   os2 = FT_Get_Sfnt_Table (face, ft_sfnt_os2);
-  if (os2 && os2->version != 0xFFFF && os2->yStrikeoutSize != 0)
+  if (os2 && os2->version != 0xFFFF)
     {
       FT_Fixed ft_thickness, ft_position;
 
@@ -388,12 +419,14 @@ get_face_metrics (PangoFcFont      *fcfont,
       ft_position = FT_MulFix (os2->yStrikeoutPosition, face->size->metrics.y_scale);
       metrics->strikethrough_position = PANGO_UNITS_26_6 (ft_position);
     }
-  else
+
+  if (metrics->strikethrough_thickness == 0 || metrics->strikethrough_position == 0)
     {
       metrics->strikethrough_thickness = metrics->underline_thickness;
       metrics->strikethrough_position = (PANGO_SCALE * face->size->metrics.y_ppem) / 4;
     }
 
+
   /* If hinting is on for this font, quantize the underline and strikethrough position
    * to integer values.
    */
@@ -403,8 +436,13 @@ get_face_metrics (PangoFcFont      *fcfont,
 				    &metrics->underline_position);
       pango_quantize_line_geometry (&metrics->strikethrough_thickness,
 				    &metrics->strikethrough_position);
+
+      /* Quantizing may have pushed underline_position to 0.  Not good */
+      if (metrics->underline_position == 0)
+	metrics->underline_position = - metrics->underline_thickness;
     }
 
+
   PANGO_FC_FONT_UNLOCK_FACE (fcfont);
 }
 
@@ -441,7 +479,7 @@ pango_fc_font_create_metrics_for_context (PangoFcFont   *fcfont,
   PangoRectangle extents;
   PangoLanguage *language = pango_context_get_language (context);
   const char *sample_str = pango_language_get_sample_string (language);
-  PangoFontDescription *desc = pango_font_describe_with_absolute_size (fcfont);
+  PangoFontDescription *desc = pango_font_describe_with_absolute_size (PANGO_FONT (fcfont));
 
   metrics = pango_font_metrics_new ();
 
@@ -560,14 +598,19 @@ pango_fc_font_real_get_glyph (PangoFcFont *font,
   if (entry->ch != wc)
     {
       face = PANGO_FC_FONT_LOCK_FACE (font);
-      index = FcFreeTypeCharIndex (face, wc);
-      if (index > (FT_UInt)face->num_glyphs)
-	index = 0;
+      if (G_LIKELY (face))
+        {
+	  index = FcFreeTypeCharIndex (face, wc);
+	  if (index > (FT_UInt)face->num_glyphs)
+	    index = 0;
+
+	  PANGO_FC_FONT_UNLOCK_FACE (font);
+	}
+      else
+        index = 0;
 
       entry->ch = wc;
       entry->glyph = index;
-
-      PANGO_FC_FONT_UNLOCK_FACE (font);
     }
 
   return entry->glyph;
@@ -723,12 +766,13 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
   FT_Error error;
   FT_Vector kerning;
   int i;
+  gboolean hinting = font->is_hinted;
 
   g_return_if_fail (PANGO_IS_FC_FONT (font));
   g_return_if_fail (glyphs != NULL);
 
   face = PANGO_FC_FONT_LOCK_FACE (font);
-  if (!face)
+  if (G_UNLIKELY (!face))
     return;
 
   if (!FT_HAS_KERNING (face))
@@ -745,8 +789,14 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
 			      ft_kerning_default,
 			      &kerning);
 
-      if (error == FT_Err_Ok)
-	glyphs->glyphs[i-1].geometry.width += PANGO_UNITS_26_6 (kerning.x);
+      if (error == FT_Err_Ok) {
+	int adjustment = PANGO_UNITS_26_6 (kerning.x);
+
+	if (hinting)
+	  adjustment = PANGO_UNITS_ROUND (adjustment);
+
+	glyphs->glyphs[i-1].geometry.width += adjustment;
+      }
     }
 
   PANGO_FC_FONT_UNLOCK_FACE (font);
@@ -864,6 +914,12 @@ pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
   g_return_if_fail (PANGO_IS_FC_FONT (fcfont));
 
   face = PANGO_FC_FONT_LOCK_FACE (fcfont);
+  if (G_UNLIKELY (!face))
+    {
+      /* Get generic unknown-glyph extents. */
+      pango_font_get_glyph_extents (NULL, glyph, ink_rect, logical_rect);
+      return;
+    }
 
   if (glyph == PANGO_GLYPH_EMPTY)
     gm = NULL;
